@@ -3,20 +3,20 @@ use warnings;
 
 =head1 NAME
 
-Attean::API::IDPJoinPlanning - Iterative dynamic programming query planning role
+Attean::API::IDPJoinPlanner - Iterative dynamic programming query planning role
 
 =head1 VERSION
 
-This document describes Attean::API::IDPJoinPlanning version 0.009
+This document describes Attean::API::IDPJoinPlanner version 0.010
 
 =head1 SYNOPSIS
 
   extends 'Attean::QueryPlanner';
-  with 'Attean::API::IDPJoinPlanning';
+  with 'Attean::API::IDPJoinPlanner';
 
 =head1 DESCRIPTION
 
-The Attean::API::IDPJoinPlanning role provides a query planner the
+The Attean::API::IDPJoinPlanner role provides a query planner the
 C<< joins_for_plan_alternatives >> method, as well as the cost estimation
 methods that consume the L<Attean::API::CostPlanner> role.
 
@@ -32,7 +32,7 @@ methods that consume the L<Attean::API::CostPlanner> role.
 
 =cut
 
-package Attean::API::QueryPlanner 0.009 {
+package Attean::API::QueryPlanner 0.010 {
 	use Moo::Role;
 	use Types::Standard qw(CodeRef);
 	use namespace::clean;
@@ -40,7 +40,7 @@ package Attean::API::QueryPlanner 0.009 {
 	requires 'plan_for_algebra'; # plan_for_algebra($algebra, $model, \@default_graphs)
 }
 
-package Attean::API::CostPlanner 0.009 {
+package Attean::API::CostPlanner 0.010 {
 	use Moo::Role;
 	use Types::Standard qw(CodeRef);
 	use namespace::clean;
@@ -61,13 +61,13 @@ package Attean::API::CostPlanner 0.009 {
 	}
 }
 
-package Attean::API::JoinPlanner 0.009 {
+package Attean::API::JoinPlanner 0.010 {
 	use Moo::Role;
 	use namespace::clean;
 	requires 'joins_for_plan_alternatives';
 }
 
-package Attean::API::NaiveJoinPlanner 0.009 {
+package Attean::API::NaiveJoinPlanner 0.010 {
 	use Moo::Role;
 	use Math::Cartesian::Product;
 	use namespace::clean;
@@ -95,12 +95,13 @@ package Attean::API::NaiveJoinPlanner 0.009 {
 	}
 }
 
-package Attean::API::SimpleCostPlanner 0.009 {
+package Attean::API::SimpleCostPlanner 0.010 {
 	use Moo::Role;
 	use namespace::clean;
 	use Types::Standard qw(Int);
 	use Scalar::Util qw(blessed);
 	with 'Attean::API::CostPlanner';
+	with 'MooX::Log::Any';
 
 	has 'keep' => (is => 'ro', isa => Int, default => 5);
 	
@@ -133,8 +134,9 @@ package Attean::API::SimpleCostPlanner 0.009 {
 			return $plan->cost;
 		} else {
 			if ($model->does('Attean::API::CostPlanner')) {
-				if (defined(my $cost = $model->cost_for_plan($plan, $model))) {
+				if (defined(my $cost = $model->cost_for_plan($plan, $self))) {
 					$plan->cost($cost);
+					$self->log->info('Model \''.ref($model).'\' did cost planning for \''.ref($plan).'\' and got cost '.$cost);
 					return $cost;
 				}
 			}
@@ -157,7 +159,13 @@ package Attean::API::SimpleCostPlanner 0.009 {
 				} else {
 					$cost	= $lcost * $rcost;
 				}
+				
+				# a cartesian nested loop join is bad, but the algorithm already
+				# has to check for all possible joins, so it's not as bad as
+				# a cartesian hash join (below)
+				$cost	*= 10 unless ($plan->children_are_variable_connected);
 			} elsif ($plan->isa('Attean::Plan::HashJoin')) {
+				my $joined		= $plan->children_are_variable_connected;
 				my $lcost		= $self->cost_for_plan($children[0], $model);
 				my $rcost		= $self->cost_for_plan($children[1], $model);
 				if ($lcost == 0) {
@@ -167,6 +175,8 @@ package Attean::API::SimpleCostPlanner 0.009 {
 				} else {
 					$cost	= ($lcost + $rcost);
 				}
+
+				$cost	*= 100 unless ($plan->children_are_variable_connected);
 			} elsif ($plan->isa('Attean::Plan::Service')) {
 				my $scost	= 10;
 				foreach my $c (@{ $plan->children }) {
@@ -185,12 +195,17 @@ package Attean::API::SimpleCostPlanner 0.009 {
 			}
 			
 			$plan->cost($cost);
+			if ($self->log->is_trace) {
+				$self->log->trace("Cost $cost estimated for\n".$plan->as_string);
+			} else {
+				$self->log->debug('Estimated cost for \''.ref($plan).'\' is '.$cost);
+			}
 			return $cost;
 		}
 	}
 }
 
-package Attean::API::IDPJoinPlanner 0.009 {
+package Attean::API::IDPJoinPlanner 0.010 {
 	use Moo::Role;
 	use Encode qw(encode);
 	use Attean::RDF;
@@ -313,6 +328,12 @@ package Attean::API::IDPJoinPlanner 0.009 {
 		my @plans		= @{ shift || [] };
 		no  sort 'stable';
 		my @sorted	= map { $_->[1] } sort { $a->[0] <=> $b->[0] } map { [$self->cost_for_plan($_, $model), $_] } @plans;
+		if ($self->log->is_trace) {
+			$self->log->trace('============= Plan iteration separator ==============');
+			foreach my $plan (@sorted){
+				$self->log->trace("Cost: " . $self->cost_for_plan($plan, $model) . " for plan:\n". $plan->as_string);
+			}
+		}
 		return splice(@sorted, 0, 5);
 	}
 	
@@ -331,6 +352,7 @@ package Attean::API::IDPJoinPlanner 0.009 {
 			if ($model->does('Attean::API::CostPlanner')) {
 				if (defined(my $cost = $model->cost_for_plan($plan, $model))) {
 					$plan->cost($cost);
+					$self->log->info('Model \''.ref($model).'\' did cost planning for \''.ref($plan).'\' and got cost '.$cost);
 					return $cost;
 				}
 			}
@@ -531,7 +553,7 @@ Gregory Todd Williams  C<< <gwilliams@cpan.org> >>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2014 Gregory Todd Williams.
+Copyright (c) 2014--2016 Gregory Todd Williams.
 This program is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself.
 
