@@ -7,7 +7,7 @@ Attean::Plan - Representation of SPARQL query plan operators
 
 =head1 VERSION
 
-This document describes Attean::Plan version 0.009
+This document describes Attean::Plan version 0.010
 
 =head1 SYNOPSIS
 
@@ -31,7 +31,7 @@ Evaluates a quad pattern against the model.
 
 =cut
 
-package Attean::Plan::Quad 0.009 {
+package Attean::Plan::Quad 0.010 {
 	use Moo;
 	use Scalar::Util qw(blessed reftype);
 	use Types::Standard qw(ConsumerOf ArrayRef);
@@ -136,7 +136,7 @@ Evaluates a join (natural-, anti-, or left-) using a nested loop.
 
 =cut
 
-package Attean::Plan::NestedLoopJoin 0.009 {
+package Attean::Plan::NestedLoopJoin 0.010 {
 	use Moo;
 	use List::MoreUtils qw(all);
 	use namespace::clean;
@@ -223,7 +223,7 @@ Evaluates a join (natural-, anti-, or left-) using a hash join.
 
 =cut
 
-package Attean::Plan::HashJoin 0.009 {
+package Attean::Plan::HashJoin 0.010 {
 	use Moo;
 	use List::MoreUtils qw(all);
 	use namespace::clean;
@@ -278,16 +278,61 @@ package Attean::Plan::HashJoin 0.009 {
 			my @vars	= @{ $self->join_variables };
 			my $rhs		= $children[1]->();
 			while (my $r = $rhs->next()) {
-				my $key	= join(',', map { ref($_) ? $_->as_string : '' } map { $r->value($_) } @vars);
-				push(@{ $hash{$key} }, $r);
+				my $has_unbound_right_join_var	= 0;
+				my @values;
+				foreach my $var (@vars) {
+					my $value	= $r->value($var);
+					unless (defined($value)) {
+						$has_unbound_right_join_var++;
+					}
+					push(@values, $value);
+				}
+				if ($has_unbound_right_join_var) {
+					# this is a RHS row that doesn't have a term bound to one of the join variables.
+					# this will make it impossible to compute the proper hash key to access the row bucket,
+					# so we add this row to the null bucket (hash key '') which we try to join all LHS rows
+					# against.
+					push(@{ $hash{''} }, $r);
+				} else {
+					my $key	= join(',', map { ref($_) ? $_->as_string : '' } @values);
+					push(@{ $hash{$key} }, $r);
+				}
 			}
 			
 			my @results;
 			my $lhs		= $children[0]->();
 			while (my $l = $lhs->next()) {
 				my $seen	= 0;
-				my $key		= join(',', map { ref($_) ? $_->as_string : '' } map { $l->value($_) } @vars);
-				if (my $rows = $hash{$key}) {
+				my @values;
+				my $has_unbound_left_join_var	= 0;
+				foreach my $var (@vars) {
+					my $value	= $l->value($var);
+					unless (defined($value)) {
+						$has_unbound_left_join_var++;
+					}
+					push(@values, $value);
+				}
+				
+				my @buckets;
+				if (my $b = $hash{''}) {
+					push(@buckets, $b);
+				}
+				
+				if ($has_unbound_left_join_var) {
+					my $pattern	= join(',', map { ref($_) ? quotemeta($_->as_string) : '.*' } @values);
+					foreach my $key (keys %hash) {
+						if ($key =~ /^${pattern}$/) {
+							push(@buckets, $hash{$key});
+						}
+					}
+				} else {
+					my $key		= join(',', map { ref($_) ? $_->as_string : '' } @values);
+					if (my $rows = $hash{$key}) {
+						push(@buckets, $rows);
+					}
+				}
+				
+				foreach my $rows (@buckets) {
 					foreach my $r (@$rows) {
 						if (my $j = $l->join($r)) {
 							$seen++;
@@ -316,7 +361,7 @@ package Attean::Plan::HashJoin 0.009 {
 
 =cut
 
-package Attean::Plan::Construct 0.009 {
+package Attean::Plan::Construct 0.010 {
 	use Moo;
 	use List::MoreUtils qw(all);
 	use Types::Standard qw(Str ArrayRef ConsumerOf InstanceOf);
@@ -409,7 +454,7 @@ named variable binding.
 
 =cut
 
-package Attean::Plan::EBVFilter 0.009 {
+package Attean::Plan::EBVFilter 0.010 {
 	use Moo;
 	use Scalar::Util qw(blessed);
 	use Types::Standard qw(Str ConsumerOf);
@@ -468,7 +513,7 @@ ordering.
 
 =cut
 
-package Attean::Plan::Merge 0.009 {
+package Attean::Plan::Merge 0.010 {
 	use Moo;
 	use Scalar::Util qw(blessed);
 	use Types::Standard qw(Str ArrayRef ConsumerOf);
@@ -497,7 +542,7 @@ Evaluates a set of sub-plans, returning the union of results.
 
 =cut
 
-package Attean::Plan::Union 0.009 {
+package Attean::Plan::Union 0.010 {
 	use Moo;
 	use Scalar::Util qw(blessed);
 	use namespace::clean;
@@ -562,11 +607,12 @@ expressions, binding the produced values to new variables.
 
 =cut
 
-package Attean::Plan::Extend 0.009 {
+package Attean::Plan::Extend 0.010 {
 	use Moo;
 	use Encode;
 	use Data::UUID;
 	use URI::Escape;
+	use Data::Dumper;
 	use I18N::LangTags;
 	use POSIX qw(ceil floor);
 	use Digest::SHA;
@@ -576,6 +622,7 @@ package Attean::Plan::Extend 0.009 {
 	use Types::Standard qw(ConsumerOf InstanceOf HashRef);
 	use namespace::clean;
 
+	with 'MooX::Log::Any';
 	with 'Attean::API::BindingSubstitutionPlan', 'Attean::API::UnaryQueryTree';
 	has 'expressions' => (is => 'ro', isa => HashRef[ConsumerOf['Attean::API::Expression']], required => 1);
 	
@@ -670,9 +717,7 @@ package Attean::Plan::Extend 0.009 {
 				}
 				return Attean::Literal->new(value => $num, datatype => $expr->datatype);
 			}
-			use Data::Dumper;
-			warn "Cast as $datatype: " . $term->as_string;
-			warn 'Cast expression unimplemented: ' . Dumper($expr);
+			$self->log->warn("Cast expression unimplemented for $datatype: " . Dumper($expr));
 		} elsif ($expr->isa('Attean::ValueExpression')) {
 			my $node	= $expr->value;
 			if ($node->does('Attean::API::Variable')) {
@@ -741,15 +786,14 @@ package Attean::Plan::Extend 0.009 {
 				}
 			}
 			
-			use Data::Dumper;
-			warn "Binary operator $op expression evaluation unimplemented: " . Dumper($expr);
+			$self->log->warn("Binary operator $op expression evaluation unimplemented: " . Dumper($expr));
 			die "Expression evaluation unimplemented: " . $expr->as_string;
 		} elsif ($expr->isa('Attean::FunctionExpression')) {
 			my $func	= $expr->operator;
 			if ($func eq 'IF') {
 				my ($check, @children)	= @{ $expr->children };
 				my ($term)		= $self->evaluate_expression($model, $check, $r);
-				warn $@ if $@;
+				$self->log->warn($@) if ($@);
 				my $expr	= $children[ (blessed($term) and $term->ebv) ? 0 : 1 ];
 				my $value	= $self->evaluate_expression($model, $expr, $r);
 # 				warn '############# ' . $value->as_string;
@@ -1073,7 +1117,7 @@ package Attean::Plan::Extend 0.009 {
 					return Attean::Blank->new();
 				}
 			} else {
-				warn "Expression evaluation unimplemented: " . $expr->as_string;
+				$self->log->warn("Expression evaluation unimplemented: " . $expr->as_string);
 				die "Expression evaluation unimplemented: " . $expr->as_string;
 			}
 		} elsif ($expr->isa('Attean::ExistsPlanExpression')) {
@@ -1088,7 +1132,7 @@ package Attean::Plan::Extend 0.009 {
 			
 			return $found ? Attean::Literal->true : Attean::Literal->false;
 		} else {
-			warn "Expression evaluation unimplemented: " . $expr->as_string;
+			$self->log->warn("Expression evaluation unimplemented: " . $expr->as_string);
 			die "Expression evaluation unimplemented: " . $expr->as_string;
 		}
 	}
@@ -1154,7 +1198,7 @@ hash of already-seen results.
 
 =cut
 
-package Attean::Plan::HashDistinct 0.009 {
+package Attean::Plan::HashDistinct 0.010 {
 	use Moo;
 	use namespace::clean;
 	
@@ -1182,7 +1226,7 @@ filtering out sequential duplicates.
 
 =cut
 
-package Attean::Plan::Unique 0.009 {
+package Attean::Plan::Unique 0.010 {
 	use Moo;
 	use namespace::clean;
 	
@@ -1217,7 +1261,7 @@ number of results ("offset") and limiting the total number of returned results
 
 =cut
 
-package Attean::Plan::Slice 0.009 {
+package Attean::Plan::Slice 0.010 {
 	use Moo;
 	use Types::Standard qw(Int);
 	use namespace::clean;
@@ -1258,7 +1302,7 @@ of variable bindings in each result.
 
 =cut
 
-package Attean::Plan::Project 0.009 {
+package Attean::Plan::Project 0.010 {
 	use Moo;
 	with 'Attean::API::BindingSubstitutionPlan', 'Attean::API::UnaryQueryTree';
 	use Types::Standard qw(ArrayRef ConsumerOf);
@@ -1333,7 +1377,7 @@ sorting is applied.
 
 =cut
 
-package Attean::Plan::OrderBy 0.009 {
+package Attean::Plan::OrderBy 0.010 {
 	use Moo;
 	use Types::Standard qw(HashRef ArrayRef InstanceOf Bool Str);
 	use namespace::clean;
@@ -1397,7 +1441,7 @@ Evaluates a SPARQL query against a remove endpoint.
 
 =cut
 
-package Attean::Plan::Service 0.009 {
+package Attean::Plan::Service 0.010 {
 	use Moo;
 	use Types::Standard qw(ConsumerOf Bool Str);
 	use namespace::clean;
@@ -1429,7 +1473,7 @@ Returns a constant set of results.
 
 =cut
 
-package Attean::Plan::Table 0.009 {
+package Attean::Plan::Table 0.010 {
 	use Moo;
 	use Types::Standard qw(ArrayRef ConsumerOf);
 	use namespace::clean;
@@ -1482,7 +1526,7 @@ package Attean::Plan::Table 0.009 {
 
 =cut
 
-package Attean::Plan::ALPPath 0.009 {
+package Attean::Plan::ALPPath 0.010 {
 	use Moo;
 	use Attean::TreeRewriter;
 	use Types::Standard qw(ArrayRef ConsumerOf);
@@ -1623,7 +1667,7 @@ package Attean::Plan::ALPPath 0.009 {
 	}
 }
 
-package Attean::Plan::ZeroOrOnePath 0.009 {
+package Attean::Plan::ZeroOrOnePath 0.010 {
 	use Moo;
 	use Attean::TreeRewriter;
 	use Types::Standard qw(ArrayRef ConsumerOf);
@@ -1720,7 +1764,7 @@ results were produced by evaluating the sub-plan.
 
 =cut
 
-package Attean::Plan::Exists 0.009 {
+package Attean::Plan::Exists 0.010 {
 	use Moo;
 	use Types::Standard qw(ArrayRef ConsumerOf);
 	use namespace::clean;
@@ -1750,7 +1794,7 @@ package Attean::Plan::Exists 0.009 {
 
 =cut
 
-package Attean::Plan::Aggregate 0.009 {
+package Attean::Plan::Aggregate 0.010 {
 	use Moo;
 	use Encode;
 	use Data::UUID;
@@ -1970,7 +2014,7 @@ package Attean::Plan::Aggregate 0.009 {
 # 
 # =cut
 # 
-# package Attean::Algebra::Ask 0.009 {
+# package Attean::Algebra::Ask 0.010 {
 # 	use Moo;
 # 	sub in_scope_variables { return; }
 # 	with 'Attean::API::Algebra', 'Attean::API::UnaryQueryTree';
@@ -1992,7 +2036,7 @@ package Attean::Plan::Aggregate 0.009 {
 # 
 # =cut
 # 
-# package Attean::Algebra::Construct 0.009 {
+# package Attean::Algebra::Construct 0.010 {
 # 	use Moo;
 # 	use Types::Standard qw(ArrayRef ConsumerOf);
 # 	has 'triples' => (is => 'ro', isa => ArrayRef[ConsumerOf['Attean::API::TriplePattern']]);
@@ -2019,7 +2063,7 @@ package Attean::Plan::Aggregate 0.009 {
 # 
 # =cut
 # 
-# package Attean::Algebra::Path 0.009 {
+# package Attean::Algebra::Path 0.010 {
 # 	use Moo;
 # 	use Types::Standard qw(ConsumerOf);
 # 	sub in_scope_variables {
@@ -2057,7 +2101,7 @@ package Attean::Plan::Aggregate 0.009 {
 # 
 # =cut
 # 
-# package Attean::Algebra::NegatedPropertySet 0.009 {
+# package Attean::Algebra::NegatedPropertySet 0.010 {
 # 	use Moo;
 # 	use Types::Standard qw(ArrayRef ConsumerOf);
 # 	with 'Attean::API::PropertyPath';
@@ -2078,7 +2122,7 @@ package Attean::Plan::Aggregate 0.009 {
 # 
 # =cut
 # 
-# package Attean::Algebra::PredicatePath 0.009 {
+# package Attean::Algebra::PredicatePath 0.010 {
 # 	use Moo;
 # 	use Types::Standard qw(ConsumerOf);
 # 	with 'Attean::API::PropertyPath';
@@ -2102,7 +2146,7 @@ package Attean::Plan::Aggregate 0.009 {
 # 
 # =cut
 # 
-# package Attean::Algebra::InversePath 0.009 {
+# package Attean::Algebra::InversePath 0.010 {
 # 	use Moo;
 # 	use Types::Standard qw(ConsumerOf);
 # 	with 'Attean::API::UnaryPropertyPath';
@@ -2118,7 +2162,7 @@ package Attean::Plan::Aggregate 0.009 {
 # 
 # =cut
 # 
-# package Attean::Algebra::SequencePath 0.009 {
+# package Attean::Algebra::SequencePath 0.010 {
 # 	use Moo;
 # 	with 'Attean::API::NaryPropertyPath';
 # 	sub separator { return "/" }
@@ -2133,7 +2177,7 @@ package Attean::Plan::Aggregate 0.009 {
 # 
 # =cut
 # 
-# package Attean::Algebra::AlternativePath 0.009 {
+# package Attean::Algebra::AlternativePath 0.010 {
 # 	use Moo;
 # 	with 'Attean::API::NaryPropertyPath';
 # 	sub separator { return "|" }
@@ -2148,7 +2192,7 @@ package Attean::Plan::Aggregate 0.009 {
 # 
 # =cut
 # 
-# package Attean::Algebra::ZeroOrMorePath 0.009 {
+# package Attean::Algebra::ZeroOrMorePath 0.010 {
 # 	use Moo;
 # 	use Types::Standard qw(ConsumerOf);
 # 	with 'Attean::API::UnaryPropertyPath';
@@ -2164,7 +2208,7 @@ package Attean::Plan::Aggregate 0.009 {
 # 
 # =cut
 # 
-# package Attean::Algebra::OneOrMorePath 0.009 {
+# package Attean::Algebra::OneOrMorePath 0.010 {
 # 	use Moo;
 # 	use Types::Standard qw(ConsumerOf);
 # 	with 'Attean::API::UnaryPropertyPath';
@@ -2180,7 +2224,7 @@ package Attean::Plan::Aggregate 0.009 {
 # 
 # =cut
 # 
-# package Attean::Algebra::ZeroOrOnePath 0.009 {
+# package Attean::Algebra::ZeroOrOnePath 0.010 {
 # 	use Moo;
 # 	use Types::Standard qw(ConsumerOf);
 # 	with 'Attean::API::UnaryPropertyPath';
@@ -2215,7 +2259,7 @@ Gregory Todd Williams  C<< <gwilliams@cpan.org> >>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2014 Gregory Todd Williams.
+Copyright (c) 2014--2016 Gregory Todd Williams.
 This program is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself.
 
