@@ -1,7 +1,47 @@
+=head1 NAME
+
+AtteanX::Parser::RDFXML - RDF/XML Parser
+
+=head1 VERSION
+
+This document describes AtteanX::Parser::RDFXML version 0.010
+
+=head1 SYNOPSIS
+
+ use Attean;
+ my $parser = Attean->get_parser('RDFXML')->new();
+ $parser->parse_list_from_io( $fh );
+
+=head1 DESCRIPTION
+
+...
+
+=head1 ATTRIBUTES
+
+=over 4
+
+=item C<< canonical_media_type >>
+
+=item C<< media_types >>
+
+=item C<< file_extensions >>
+
+=item C<< bnode_prefix >>
+
+A string prefix for identifiers generated for blank nodes.
+
+=back
+
+=head1 METHODS
+
+=over 4
+
+=cut
+
 use v5.14;
 use warnings;
 
-package AtteanX::Parser::RDFXML 0.009 {
+package AtteanX::Parser::RDFXML 0.010 {
 	use Moo;
 	use Types::Standard qw(Str Object);
 	use Attean;
@@ -14,10 +54,6 @@ package AtteanX::Parser::RDFXML 0.009 {
 	use Scalar::Util qw(blessed);
 	use Module::Load::Conditional qw[can_load];
 
-=head1 METHODS
-
-=over 4
-
 =item C<< canonical_media_type >>
 
 Returns the canonical media type for SPARQL XML: application/sparql-results+json.
@@ -78,13 +114,27 @@ the data read from the UTF-8 encoded byte string C<< $data >>.
 		my $self	= shift;
 		my $data	= shift;
 
-		my $saxhandler	= AtteanX::Parser::RDFXML::SAXHandler->new( bnode_prefix => $self->bnode_prefix, handler => $self->handler );
+		my @args;
+		if (my $map = $self->namespaces) {
+			push(@args, namespaces => $map);
+		}
+		my $saxhandler	= AtteanX::Parser::RDFXML::SAXHandler->new( bnode_prefix => $self->bnode_prefix, handler => $self->handler, @args );
 		my $p			= XML::SAX::ParserFactory->parser(Handler => $saxhandler);
 		$saxhandler->push_base( $self->base ) if ($self->has_base);
-		if (ref($data)) {
-			$p->parse_file($data);
-		} else {
-			$p->parse_string($data);
+		eval {
+			if (ref($data)) {
+				$p->parse_file($data);
+			} else {
+				$p->parse_string($data);
+			}
+		};
+		
+		if ($@) {
+			if ($@ =~ /no element found at line 1, column 0, byte 0/) {
+				# silence XML::Parser output on empty input
+			} else {
+				die $@;
+			}
 		}
 		my $nodes	= $saxhandler->{nodes};
 		if ($nodes and scalar(@$nodes)) {
@@ -657,4 +707,19 @@ __END__
 
 =back
 
+=head1 BUGS
+
+Please report any bugs or feature requests to through the GitHub web interface
+at L<https://github.com/kasei/perlrdf/issues>.
+
+=head1 AUTHOR
+
+Gregory Todd Williams  C<< <gwilliams@cpan.org> >>
+
+=head1 COPYRIGHT
+
+Copyright (c) 2014--2016 Gregory Todd Williams. This
+program is free software; you can redistribute it and/or modify it under
+the same terms as Perl itself.
+
 =cut
