@@ -7,7 +7,7 @@ AtteanX::Parser::SPARQLTSV - SPARQL Results TSV Parser
 
 =head1 VERSION
 
-This document describes AtteanX::Parser::SPARQLTSV version 0.009
+This document describes AtteanX::Parser::SPARQLTSV version 0.010
 
 =head1 SYNOPSIS
 
@@ -17,13 +17,25 @@ This document describes AtteanX::Parser::SPARQLTSV version 0.009
 
 ...
 
+=head1 ATTRIBUTES
+
+=over 4
+
+=item C<< canonical_media_type >>
+
+=item C<< media_types >>
+
+=item C<< file_extensions >>
+
+=back
+
 =head1 METHODS
 
 =over 4
 
 =cut
 
-package AtteanX::Parser::SPARQLTSV 0.009 {
+package AtteanX::Parser::SPARQLTSV 0.010 {
 	use utf8;
 	use Moo;
 	use Attean;
@@ -38,12 +50,6 @@ package AtteanX::Parser::SPARQLTSV 0.009 {
 		return [qw(text/tab-separated-values)];
 	}
 	
-=item C<< file_extensions >>
-
-Returns a list of file extensions that may be parsed with the parser.
-
-=cut
-
 	sub file_extensions { return [qw(tsv)] }
 
 	with 'Attean::API::ResultParser', 'Attean::API::PullParser', 'Attean::API::Parser';
@@ -100,7 +106,7 @@ the data read from the L<IO::Handle> object C<< $fh >>.
 				if (length($string)) {
 					my $var	= $vars[$i];
 					my $bytes	= encode('UTF-8', $string, Encode::FB_CROAK);
-					my $term	= $parser->parse_term_from_string($bytes);
+					my $term	= $parser->parse_term_from_bytes($bytes);
 					if ($term) {
 						$binding{ $var }	= $term;
 					}
@@ -133,7 +139,7 @@ Gregory Todd Williams  C<< <gwilliams@cpan.org> >>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2006-2012 Gregory Todd Williams. This
+Copyright (c) 2014--2016 Gregory Todd Williams. This
 program is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself.
 
