@@ -4,7 +4,7 @@ AtteanX::Serializer::SPARQL - SPARQL Serializer
 
 =head1 VERSION
 
-This document describes AtteanX::Serializer::SPARQL version 0.009
+This document describes AtteanX::Serializer::SPARQL version 0.010
 
 =head1 SYNOPSIS
 
@@ -16,6 +16,20 @@ This document describes AtteanX::Serializer::SPARQL version 0.009
 
 ...
 
+=head1 ATTRIBUTES
+
+=over 4
+
+=item C<< canonical_media_type >>
+
+=item C<< media_types >>
+
+=item C<< handled_type >>
+
+=item C<< file_extensions >>
+
+=back
+
 =head1 METHODS
 
 =over 4
@@ -25,7 +39,7 @@ This document describes AtteanX::Serializer::SPARQL version 0.009
 use v5.14;
 use warnings;
 
-package AtteanX::Serializer::SPARQL 0.009 {
+package AtteanX::Serializer::SPARQL 0.010 {
 	use Moo;
 	use Data::Dumper;
 	use Encode qw(encode);
@@ -74,13 +88,16 @@ L<IO::Handle> object C<< $fh >>.
 				if ($type == RBRACE) {
 					$io->print("\n");
 					$newline	= 1;
-					$indent--;
 				} elsif ($type == KEYWORD and $t->value =~ /^(BASE|PREFIX|SELECT|ASK|CONSTRUCT|DESCRIBE)$/) {
 					$io->print("\n");
 					$newline	= 1;
 				}
 			}
 			
+			if ($type == RBRACE) {
+				$indent--;
+			}
+			
 			if ($newline) {
 				$io->print('    ' x $indent);
 				$newline	= 0;
@@ -96,6 +113,10 @@ L<IO::Handle> object C<< $fh >>.
 				my $args	= $t->args;
 				$io->print(join('', @$args));
 				$need_space++;
+			} elsif ($type == BNODE) {
+				$io->print('_:');
+				$io->print($t->value);
+				$need_space++;
 			} elsif ($type == IRI) {
 				# TODO: escape
 				$io->print('<');
@@ -138,6 +159,10 @@ L<IO::Handle> object C<< $fh >>.
 				$io->print('?');
 				$io->print($t->value);
 				$need_space++;
+			} elsif ($type == ANON) {
+				$io->print('_:');
+				$io->print($t->value);
+				$need_space++;
 			} elsif ($type == A) {
 				$io->print('a');
 				$need_space++;
@@ -153,12 +178,14 @@ L<IO::Handle> object C<< $fh >>.
 				$need_space++;
 			}
 			
-			if ($type == DOT and $semicolon) {
-				$indent--;
+			if ($type == DOT) {
+				if ($semicolon) {
+					$indent--;
+					$semicolon	= 0;
+				}
 				$need_space	= 0;
 				$io->print("\n");
 				$newline	= 1;
-				$semicolon	= 0;
 			} elsif ($type == LBRACE) {
 				$io->print("\n");
 				$need_space	= 0;
@@ -168,13 +195,15 @@ L<IO::Handle> object C<< $fh >>.
 				$io->print("\n");
 				$need_space	= 0;
 				$newline	= 1;
-				$semicolon	= 1;
 				unless ($semicolon) {
 					$indent++;
 				}
+				$semicolon	= 1;
 			}
 		}
-		$io->print("\n");
+		unless ($newline) {
+			$io->print("\n");
+		}
 		return;
 	}
 	
@@ -190,7 +219,7 @@ and returns the serialization as a UTF-8 encoded byte string.
 		my $iter	= shift;
 		my $data	= '';
 		open(my $fh, '>:utf8', \$data);
-		$self->serialize_iter_to_io($fh);
+		$self->serialize_iter_to_io($fh, $iter);
 		close($fh);
 		return $data;
 	}
@@ -213,7 +242,7 @@ Gregory Todd Williams  C<< <gwilliams@cpan.org> >>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2006-2012 Gregory Todd Williams. This
+Copyright (c) 2014--2016 Gregory Todd Williams. This
 program is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself.
 
